package com.ruoyi.system.service;

import com.ruoyi.system.domain.PoUser;

import java.util.List;

/**
 * 用户Service接口
 *
 * @author ruoyi
 * @date 2023-01-17
 */
public interface IPoUserService
{
    /**
     * 查询用户
     *
     * @param userId 用户主键
     * @return 用户
     */
    public PoUser selectPoUserByUserId(Long userId);

    /**
     * 查询用户列表
     *
     * @param poUser 用户
     * @return 用户集合
     */
    public List<PoUser> selectPoUserList(PoUser poUser);

    /**
     * 新增用户
     *
     * @param poUser 用户
     * @return 结果
     */
    public int insertPoUser(PoUser poUser);

    /**
     * 修改用户
     *
     * @param poUser 用户
     * @return 结果
     */
    public int updatePoUser(PoUser poUser);

    /**
     * 批量删除用户
     *
     * @param userIds 需要删除的用户主键集合
     * @return 结果
     */
    public int deletePoUserByUserIds(Long[] userIds);

    /**
     * 删除用户信息
     *
     * @param userId 用户主键
     * @return 结果
     */
    public int deletePoUserByUserId(Long userId);

    /**
     * 检验用户账号是否存在
     * @param poUser
     * @return
     */
    String checkUserNameUnique(PoUser poUser);

    /**
     * 检验用户手机号是否存在
     * @param poUser
     * @return
     */
    String checkPhoneUnique(PoUser poUser);

    /**
     * 检验邮箱是否存在
     * @param poUser
     * @return
     */
    String checkEmailUnique(PoUser poUser);

    /**
     * 检验当前用户是否可以操作
     * @param poUser
     */
    void checkUserAllowed(PoUser poUser);

    /**
     * 检验用户是否有访问数据的权限
     * @param userId
     */
    void checkUserDataScope(Long userId);

    /**
     * 重置密码
     * @param poUser
     * @return
     */
    int resetPwd(PoUser poUser);

    /**
     * 状态修改
     * @param poUser
     * @return
     */
    int updateUserStatus(PoUser poUser);
}
